#
# Copyright 2014, 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Module to rename blocks """


import os
import re
import logging

from .base import get_block_candidates, ModTool, ModToolException, validate_name

logger = logging.getLogger(__name__)


class ModToolRename(ModTool):
    """ Rename a block in the out-of-tree module. """
    name = 'rename'
    description = 'Rename a block inside a module.'

    def __init__(self, blockname=None, new_name=None, **kwargs):
        ModTool.__init__(self, blockname, **kwargs)
        self.info['oldname'] = blockname
        self.info['newname'] = new_name

    def validate(self):
        """ Validates the arguments """
        ModTool._validate(self)
        if not self.info['oldname']:
            raise ModToolException('Old block name (blockname) not specified.')
        validate_name('old block', self.info['oldname'])
        block_candidates = get_block_candidates()
        if self.info['oldname'] not in block_candidates:
            choices = [x for x in block_candidates if self.info['oldname'] in x]
            if len(choices)>0:
                print("Suggested alternatives: "+str(choices))
            raise ModToolException("Blockname for renaming does not exists!")
        if not self.info['newname']:
            raise ModToolException('New blockname (new_name) not specified.')
        validate_name('new block', self.info['newname'])

    def assign(self):
        self.info['fullnewname'] = self.info['modname'] + '_' + self.info['newname']

    def run(self):
        """ Go, go, go. """
        # This portion will be covered by the CLI
        if not self.cli:
            self.validate()
            self.assign()
        module = self.info['modname']
        oldname = self.info['oldname']
        newname = self.info['newname']
        logger.info(f"In module '{module}' rename block '{oldname}' to '{newname}'")
        self._run_grc_rename(self.info['modname'], oldname, newname)
        self._run_python_qa(self.info['modname'], oldname, newname)
        self._run_python(self.info['modname'], oldname, newname)
        self._run_lib(self.info['modname'], oldname, newname)
        self._run_include(self.info['modname'], oldname, newname)
        self._run_pybind(self.info['modname'], oldname, newname)
        return

    def _run_lib(self, module, old, new):
        ccfile = './lib/' + old + '_impl.cc'
        if not os.path.isfile(ccfile):  # in case it is a 'noblock'
            ccfile = './lib/' + old + '.cc'
        hfile = './lib/' + old + '_impl.h'
        self._run_file_replace(ccfile, old, new)
        self._run_file_replace(hfile, old, new)
        self._run_file_replace(hfile, old.upper(), new.upper())  # take care of include guards
        self._run_cmakelists('./lib/', old, new)
        self._run_file_rename('./lib/', old, new)
        self._run_cpp_qa(module, old, new)

    def _run_cpp_qa(self, module, old, new):
        path = './lib/'
        filename = 'qa_' + module + '.cc'
        nsubs = self._run_file_replace(path + filename, old, new)
        if nsubs > 0:
            logger.info("C++ QA code detected, renaming...")
            filename = 'qa_' + old + '.cc'
            self._run_file_replace(path + filename, old, new)
            filename = 'qa_' + old + '.h'
            self._run_file_replace(path + filename, old, new)
            self._run_file_replace(path + filename, old.upper(), new.upper())
            self._run_file_rename(path, 'qa_' + old, 'qa_' + new)
        else:
            logger.info("No C++ QA code detected, skipping...")

    def _run_include(self, module, old, new):
        path = './include/' + module + '/'
        filename = path + old + '.h'
        self._run_file_replace(filename, old, new)
        self._run_file_replace(filename, old.upper(), new.upper())  # take care of include guards
        self._run_cmakelists(path, old, new)
        self._run_file_rename(path, old, new)

    def _run_python(self, module, old, new):
        path = './python/'
        filename = '__init__.py'
        nsubs = self._run_file_replace(path + filename, old, new)
        if nsubs > 0:
            logger.info("Python block detected, renaming...")
            filename = old + '.py'
            self._run_file_replace(path + filename, old, new)
            self._run_cmakelists(path, old, new)
            self._run_file_rename(path, old, new)
        else:
            logger.info("Not a Python block, nothing to do here...")

    def _run_pybind(self, module, old, new):
        path = './python/bindings/'
        filename = path + old + '_python.cc'
        self._run_file_replace(filename, old, new)
        self._run_file_rename(path, old, new)
        self._run_cmakelists(path, old, new)
        # update the hash in the new file
        import hashlib
        hasher = hashlib.md5()
        header_filename = './include/' + module + '/' + new + '.h' # note this requires _run_pybind to be called after _run_include
        with open(header_filename, 'rb') as file_in:
            buf = file_in.read()
            hasher.update(buf)
        newhash = hasher.hexdigest()
        newfilename = path + new + '_python.cc'
        with open(newfilename) as f:
            file_txt = f.read()
        m = re.search(r'BINDTOOL_HEADER_FILE_HASH\(([^\s]*)\)', file_txt)
        if (m):
            oldhash = m.group(1)
            file_txt = re.sub(oldhash, newhash, file_txt)
        with open(newfilename, "w") as f:
            f.write(file_txt)
       
        path = './python/bindings/docstrings/'
        filename = path + old + '_pydoc_template.h'
        self._run_file_replace(filename, old, new)
        self._run_file_rename(path, old, new)

    def _run_python_qa(self, module, old, new):
        new = 'qa_' + new
        old = 'qa_' + old
        filename = './python/' + old + '.py'
        self._run_file_replace(filename, old, new)
        self._run_cmakelists('./python/', old, new)
        self._run_file_rename('./python/', old, new)

    def _run_grc_rename(self, module, old, new):
        grcfile = './grc/' + module + '_' + old + '.yml'
        self._run_file_replace(grcfile, old, new)
        self._run_cmakelists('./grc/', old, new)
        self._run_file_rename('./grc/', module + '_' + old, module + '_' + new)

    def _run_cmakelists(self, path, first, second):
        filename = path + 'CMakeLists.txt'
        nsubs = self._run_file_replace(filename, first, second)
        if nsubs < 1:
            logger.info(f"'{first}' wasn't in '{filename}'.")

    def _run_file_rename(self, path, old, new):
        files = os.listdir(path)
        for file in files:
            if file.find(old) > -1 and file.find(old) < 3:
                nl = file.replace(old, new)
                src = path + file
                dst = path + nl
                logger.info(f"Renaming file '{src}' to '{dst}'.")
                os.rename(src, dst)

    def _run_file_replace(self, filename, old, new):
        if not os.path.isfile(filename):
            return False
        else:
            logger.info(f"In '{filename}' renaming occurrences of '{old}' to '{new}'")

        with open(filename) as f:
            cfile = f.read()
        (cfile, nsubs) = re.subn(old, new, cfile)

        with open(filename, 'w') as f:
            f.write(cfile)
        self.scm.mark_file_updated(filename)
        return nsubs



#!D:\bld\gnuradio_1628795209068\_h_env\python.exe
#
# Copyright 2012, 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" A tool for editing GNU Radio out-of-tree modules. """


from gnuradio.modtool.cli.base import cli


if __name__ == '__main__':
    try:
        cli()
    except KeyboardInterrupt:
        pass



#!D:\bld\gnuradio_1628795209068\_h_env\python.exe
#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import sys
from argparse import ArgumentParser

import pmt
from gnuradio.blocks import parse_file_metadata

def main(filename, detached=False):
    handle = open(filename, "rb")

    nheaders = 0
    nread = 0
    while(True):
        # read out next header bytes
        hdr_start = handle.tell()
        header_str = handle.read(parse_file_metadata.HEADER_LENGTH)
        if(len(header_str) == 0):
            break

        # Convert from string to PMT (should be a dictionary)
        try:
            header = pmt.deserialize_str(header_str)
        except RuntimeError:
            sys.stderr.write("Could not deserialize header: invalid or corrupt data file.\n")
            sys.exit(1)

        print("HEADER {0}".format(nheaders))
        info = parse_file_metadata.parse_header(header, True)

        if(info["extra_len"] > 0):
            extra_str = handle.read(info["extra_len"])
            if(len(extra_str) == 0):
                break

            try:
                extra = pmt.deserialize_str(extra_str)
            except RuntimeError:
                sys.stderr.write("Could not deserialize extras: invalid or corrupt data file.\n")
                sys.exit(1)

            print("\nExtra Header:")
            extra_info = parse_file_metadata.parse_extra_dict(extra, info, True)

        nheaders += 1
        nread += parse_file_metadata.HEADER_LENGTH + info["extra_len"]
        if(not detached):
            nread += info['nbytes']
        handle.seek(nread, 0)
        print("\n\n")


if __name__ == "__main__":
    description = "Read in a GNU Radio file with meta data, extracts the header and prints it."

    parser = ArgumentParser(conflict_handler="resolve", description=description)
    parser.add_argument("-D", "--detached", action="store_true",
                      help="Used if header is detached.")
    parser.add_argument("file", metavar="FILE",
                      help="Input file");
    args = parser.parse_args()

    main(args.file, args.detached)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import numpy
from .constellation_map_generator import constellation_map_generator

'''
Note on the naming scheme. Each constellation is named using a prefix
for the type of constellation, the order of the constellation, and a
distinguishing feature, which comes in three modes:

- No extra feature: the basic Gray-coded constellation map; others
  will be derived from this type.
- A single number: an indexed number to uniquely identify different
  constellation maps.
- 0xN_x0_x1..._xM: A permutation of the base constellation, explained
  below.

For rectangular constellations (BPSK, QPSK, QAM), we can define a
hyperspace and look for all symmetries. This is also known as the
automorphism group of the hypercube, aka the hyperoctahedral
group. What this means is that we can easily define all possible
rotations in terms of the first base mapping by creating the mapping:

  f(x) = k XOR pi(x)

The x is the bit string for the symbol we are altering. Then k is a
bit string of n bits where n is the number of bits per symbol in the
constellation (e.g., 2 for QPSK or 6 for QAM64). The pi is a
permutation function specified as pi_0, pi_1..., pi_n-1. This permutes
the bits from the base constellation symbol to a new code, which is
then xor'd by k.

The value of k is from 0 to 2^n-1 and pi is a list of all bit
positions.

The total number of Gray coded modulations is (2^n)*(n!).

We create aliases for all possible naming schemes for the
constellations. So if a hyperoctahedral group is defined, we also set
this function equal to a function name using a unique ID number, and
we always select one rotation as our basic rotation that the other
rotations are based off of.
'''

# BPSK Constellation Mappings

def psk_2_0x0():
    '''
    0 | 1
    '''
    const_points = [-1, 1]
    symbols = [0, 1]
    return (const_points, symbols)
psk_2 = psk_2_0x0  # Basic BPSK rotation
psk_2_0 = psk_2    # First ID for BPSK rotations

def psk_2_0x1():
    '''
    1 | 0
    '''
    const_points = [-1, 1]
    symbols = [1, 0]
    return (const_points, symbols)
psk_2_1 = psk_2_0x1


############################################################
# BPSK Soft bit LUT generators
############################################################

def sd_psk_2_0x0(x, Es=1):
    '''
    0 | 1
    '''
    x_re = x.real
    dist = Es*numpy.sqrt(2)
    return [dist*x_re,]
sd_psk_2 = sd_psk_2_0x0  # Basic BPSK rotation
sd_psk_2_0 = sd_psk_2    # First ID for BPSK rotations

def sd_psk_2_0x1(x, Es=1):
    '''
    1 | 0
    '''
    x_re = [x.real,]
    dist = Es*numpy.sqrt(2)
    return -dist*x_re
sd_psk_2_1 = sd_psk_2_0x1


############################################################
# QPSK Constellation Mappings
############################################################

def psk_4_0x0_0_1():
    '''
    | 10 | 11
    | -------
    | 00 | 01
    '''
    const_points = [-1-1j, 1-1j,
                    -1+1j, 1+1j]
    symbols = [0, 1, 2, 3]
    return (const_points, symbols)
psk_4 = psk_4_0x0_0_1
psk_4_0 = psk_4

def psk_4_0x1_0_1():
    '''
    | 11 | 10
    | -------
    | 01 | 00
    '''
    k = 0x1
    pi = [0, 1]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_1 = psk_4_0x1_0_1

def psk_4_0x2_0_1():
    '''
    | 00 | 01
    | -------
    | 10 | 11
    '''
    k = 0x2
    pi = [0, 1]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_2 = psk_4_0x2_0_1

def psk_4_0x3_0_1():
    '''
    | 01 | 00
    | -------
    | 11 | 10
    '''
    k = 0x3
    pi = [0, 1]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_3 = psk_4_0x3_0_1

def psk_4_0x0_1_0():
    '''
    | 01 | 11
    | -------
    | 00 | 10
    '''
    k = 0x0
    pi = [1, 0]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_4 = psk_4_0x0_1_0

def psk_4_0x1_1_0():
    '''
    | 00 | 10
    | -------
    | 01 | 11
    '''
    k = 0x1
    pi = [1, 0]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_5 = psk_4_0x1_1_0

def psk_4_0x2_1_0():
    '''
    | 11 | 01
    | -------
    | 10 | 00
    '''
    k = 0x2
    pi = [1, 0]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_6 = psk_4_0x2_1_0

def psk_4_0x3_1_0():
    '''
    | 10 | 00
    | -------
    | 11 | 01
    '''
    k = 0x3
    pi = [1, 0]
    return constellation_map_generator(psk_4()[0], psk_4()[1], k, pi)
psk_4_7 = psk_4_0x3_1_0



############################################################
# QPSK Constellation Softbit LUT generators
############################################################

def sd_psk_4_0x0_0_1(x, Es=1):
    '''
    | 10 | 11
    | -------
    | 00 | 01
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [dist*x_im, dist*x_re]
sd_psk_4 = sd_psk_4_0x0_0_1
sd_psk_4_0 = sd_psk_4

def sd_psk_4_0x1_0_1(x, Es=1):
    '''
    | 11 | 10
    | -------
    | 01 | 00
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [dist*x_im, -dist*x_re]
sd_psk_4_1 = sd_psk_4_0x1_0_1

def sd_psk_4_0x2_0_1(x, Es=1):
    '''
    | 00 | 01
    | -------
    | 10 | 11
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [-dist*x_im, dist*x_re]
sd_psk_4_2 = sd_psk_4_0x2_0_1

def sd_psk_4_0x3_0_1(x, Es=1):
    '''
    | 01 | 00
    | -------
    | 11 | 10
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [-dist*x_im, -dist*x_re]
sd_psk_4_3 = sd_psk_4_0x3_0_1

def sd_psk_4_0x0_1_0(x, Es=1):
    '''
    | 01 | 11
    | -------
    | 00 | 10
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [dist*x_re, dist*x_im]
sd_psk_4_4 = sd_psk_4_0x0_1_0

def sd_psk_4_0x1_1_0(x, Es=1):
    '''
    | 00 | 10
    | -------
    | 01 | 11
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [dist*x_re, -dist*x_im]
sd_psk_4_5 = sd_psk_4_0x1_1_0


def sd_psk_4_0x2_1_0(x, Es=1):
    '''
    | 11 | 01
    | -------
    | 10 | 00
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [-dist*x_re, dist*x_im]
sd_psk_4_6 = sd_psk_4_0x2_1_0

def sd_psk_4_0x3_1_0(x, Es=1):
    '''
    | 10 | 00
    | -------
    | 11 | 01
    '''
    x_re = x.real
    x_im = x.imag
    dist = Es*numpy.sqrt(2)
    return [-dist*x_re, -dist*x_im]
sd_psk_4_7 = sd_psk_4_0x3_1_0



#!D:\bld\gnuradio_1628795209068\_h_env\python.exe

from gnuradio import gr
from gnuradio import blocks
import argparse
from volk_test_funcs import (create_connection, new_table, replace_results,
                             helper, timeit, format_results)

try:
    from gnuradio import blocks
except ImportError:
    sys.stderr.write("This example requires gr-blocks.\n")

def multiply_const_cc(N):
    k = 3.3
    op = blocks.multiply_const_cc(k)
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_gr_complex, 1, 1)
    return tb

######################################################################

def multiply_const_ff(N):
    k = 3.3
    op = blocks.multiply_const_ff(k)
    tb = helper(N, op, gr.sizeof_float, gr.sizeof_float, 1, 1)
    return tb

######################################################################

def multiply_cc(N):
    op = blocks.multiply_cc(1)
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_gr_complex, 2, 1)
    return tb

######################################################################

def multiply_ff(N):
    op = blocks.multiply_ff()
    tb = helper(N, op, gr.sizeof_float, gr.sizeof_float, 2, 1)
    return tb

######################################################################

def add_ff(N):
    op = blocks.add_ff()
    tb = helper(N, op, gr.sizeof_float, gr.sizeof_float, 2, 1)
    return tb

######################################################################

def conjugate_cc(N):
    op = blocks.conjugate_cc()
    tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_gr_complex, 1, 1)
    return tb

######################################################################

def multiply_conjugate_cc(N):
    try:
        op = blocks.multiply_conjugate_cc()
        tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_gr_complex, 2, 1)
        return tb

    except AttributeError:
        class s(gr.hier_block2):
            def __init__(self):
                gr.hier_block2.__init__(self, "s",
                                        gr.io_signature(2, 2, gr.sizeof_gr_complex),
                                        gr.io_signature(1, 1, gr.sizeof_gr_complex))
                conj = blocks.conjugate_cc()
                mult = blocks.multiply_cc()
                self.connect((self,0), (mult,0))
                self.connect((self,1), conj, (mult,1))
                self.connect(mult, self)

        op = s()
        tb = helper(N, op, gr.sizeof_gr_complex, gr.sizeof_gr_complex, 2, 1)
        return tb


######################################################################

def run_tests(func, N, iters):
    print("Running Test: {0}".format(func.__name__))
    try:
        tb = func(N)
        t = timeit(tb, iters)
        res = format_results(func.__name__, t)
        return res
    except AttributeError:
        print("\tCould not run test. Skipping.")
        return None

def main():
    avail_tests = [multiply_const_cc,
                   multiply_const_ff,
                   multiply_cc,
                   multiply_ff,
                   add_ff,
                   conjugate_cc,
                   multiply_conjugate_cc]

    desc='Time an operation to compare with other implementations. \
          This program runs a simple GNU Radio flowgraph to test a \
          particular math function, mostly to compare the  \
          Volk-optimized implementation versus a regular \
          implementation. The results are stored to an SQLite database \
          that can then be read by volk_plot.py to plot the differences.'
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('-L', '--label', type=str,
                        required=True, default=None,
                        help='Label of database table [default: %(default)s]')
    parser.add_argument('-D', '--database', type=str,
                        default="volk_results.db",
                        help='Database file to store data in [default: %(default)s]')
    parser.add_argument('-N', '--nitems', type=float,
                        default=1e9,
                        help='Number of items per iterations [default: %(default)s]')
    parser.add_argument('-I', '--iterations', type=int,
                        default=20,
                        help='Number of iterations [default: %(default)s]')
    parser.add_argument('--tests', type=int, nargs='*',
                        choices=list(range(len(avail_tests))),
                        help='A list of tests to run; can be a single test or a \
                              space-separated list.')
    parser.add_argument('--list', action='store_true',
                        help='List the available tests')
    parser.add_argument('--all', action='store_true',
                        help='Run all tests')
    args = parser.parse_args()

    if(args.list):
        print("Available Tests to Run:")
        print("\n".join(["\t{0}: {1}".format(i,f.__name__) for i,f in enumerate(avail_tests)]))
        sys.exit(0)

    N = int(args.nitems)
    iters = args.iterations
    label = args.label

    conn = create_connection(args.database)
    new_table(conn, label)

    if args.all:
        tests = list(range(len(avail_tests)))
    else:
        tests = args.tests

    for test in tests:
        res = run_tests(avail_tests[test], N, iters)
        if res is not None:
            replace_results(conn, label, N, iters, res)

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



#!D:\bld\gnuradio_1628795209068\_h_env\python.exe
#
# Copyright 2007,2008 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio.plot_data import plot_data


def main():
    parser = plot_data.setup_options()
    args = parser.parse_args()
    plot_data(None, args.files, args)

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass




#!/usr/bin/env python
#
# Copyright 2009,2010,2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, fft, blocks

from . import optfir
from . import filter_python as filter, fft


class channelizer_ccf(gr.hier_block2):
    '''
    Make a Polyphase Filter channelizer (complex in, complex out, floating-point taps)

    This simplifies the interface by allowing a single input stream to connect to this block.
    It will then output a stream for each channel.
    '''
    def __init__(self, numchans, taps=None, oversample_rate=1, atten=100):
        gr.hier_block2.__init__(self, "pfb_channelizer_ccf",
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(numchans, numchans, gr.sizeof_gr_complex))

        self._nchans = numchans
        self._oversample_rate = oversample_rate

        if (taps is not None) and (len(taps) > 0):
            self._taps = taps
        else:
            self._taps = self.create_taps(self._nchans, atten)

        self.s2ss = blocks.stream_to_streams(gr.sizeof_gr_complex, self._nchans)
        self.pfb = filter.pfb_channelizer_ccf(self._nchans, self._taps,
                                              self._oversample_rate)
        self.connect(self, self.s2ss)

        for i in range(self._nchans):
            self.connect((self.s2ss,i), (self.pfb,i))
            self.connect((self.pfb,i), (self,i))

    def set_channel_map(self, newmap):
        self.pfb.set_channel_map(newmap)

    def set_taps(self, taps):
        self.pfb.set_taps(taps)

    def taps(self):
        return self.pfb.taps()

    def declare_sample_delay(self, delay):
        self.pfb.declare_sample_delay(delay)
    
    @staticmethod
    def create_taps(numchans, atten=100):
        # Create a filter that covers the full bandwidth of the input signal
        bw = 0.4
        tb = 0.2
        ripple = 0.1
        while True:
            try:
                taps = optfir.low_pass(1, numchans, bw, bw+tb, ripple, atten)
                return taps
            except RuntimeError:
                ripple += 0.01
                print("Warning: set ripple to %.4f dB. If this is a problem, adjust the attenuation or create your own filter taps." % (ripple))

                # Build in an exit strategy; if we've come this far, it ain't working.
                if(ripple >= 1.0):
                    raise RuntimeError("optfir could not generate an appropriate filter.")


class interpolator_ccf(gr.hier_block2):
    '''
    Make a Polyphase Filter interpolator (complex in, complex out, floating-point taps)

    The block takes a single complex stream in and outputs a single complex
    stream out. As such, it requires no extra glue to handle the input/output
    streams. This block is provided to be consistent with the interface to the
    other PFB block.
    '''
    def __init__(self, interp, taps=None, atten=100):
        gr.hier_block2.__init__(self, "pfb_interpolator_ccf",
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(1, 1, gr.sizeof_gr_complex))

        self._interp = interp
        self._taps = taps

        if (taps is not None) and (len(taps) > 0):
            self._taps = taps
        else:
            self._taps = self.create_taps(self._interp, atten)

        self.pfb = filter.pfb_interpolator_ccf(self._interp, self._taps)

        self.connect(self, self.pfb)
        self.connect(self.pfb, self)

    def set_taps(self, taps):
        self.pfb.set_taps(taps)

    def declare_sample_delay(self, delay):
        self.pfb.declare_sample_delay(delay)

    @staticmethod
    def create_taps(interp, atten):
        # Create a filter that covers the full bandwidth of the input signal
        bw = 0.4
        tb = 0.2
        ripple = 0.99
        while True:
            try:
                taps = optfir.low_pass(interp, interp, bw, bw+tb, ripple, atten)
                return taps
            except RuntimeError:
                ripple += 0.01
                print("Warning: set ripple to %.4f dB. If this is a problem, adjust the attenuation or create your own filter taps." % (ripple))

                # Build in an exit strategy; if we've come this far, it ain't working.
                if(ripple >= 1.0):
                    raise RuntimeError("optfir could not generate an appropriate filter.")



class decimator_ccf(gr.hier_block2):
    '''
    Make a Polyphase Filter decimator (complex in, complex out, floating-point taps)

    This simplifies the interface by allowing a single input stream to connect to this block.
    It will then output a stream that is the decimated output stream.
    '''
    def __init__(self, decim, taps=None, channel=0, atten=100,
                 use_fft_rotators=True, use_fft_filters=True):
        gr.hier_block2.__init__(self, "pfb_decimator_ccf",
                                gr.io_signature(1, 1, gr.sizeof_gr_complex),
                                gr.io_signature(1, 1, gr.sizeof_gr_complex))

        self._decim = decim
        self._channel = channel

        if (taps is not None) and (len(taps) > 0):
            self._taps = taps
        else:
            self._taps = self.create_taps(self._decim, atten)

        self.s2ss = blocks.stream_to_streams(gr.sizeof_gr_complex, self._decim)
        self.pfb = filter.pfb_decimator_ccf(self._decim, self._taps, self._channel,
                                            use_fft_rotators, use_fft_filters)

        self.connect(self, self.s2ss)

        for i in range(self._decim):
            self.connect((self.s2ss,i), (self.pfb,i))

        self.connect(self.pfb, self)

    def set_taps(self, taps):
        self.pfb.set_taps(taps)

    def set_channel(self, chan):
        self.pfb.set_channel(chan)

    def declare_sample_delay(self, delay):
        self.pfb.declare_sample_delay(delay)
    
    @staticmethod
    def create_taps(decim, atten=100):
        # Create a filter that covers the full bandwidth of the input signal
        bw = 0.4
        tb = 0.2
        ripple = 0.1
        while True:
            try:
                taps = optfir.low_pass(1, decim, bw, bw+tb, ripple, atten)
                return taps
            except RuntimeError:
                ripple += 0.01
                print("Warning: set ripple to %.4f dB. If this is a problem, adjust the attenuation or create your own filter taps." % (ripple))

                # Build in an exit strategy; if we've come this far, it ain't working.
                if(ripple >= 1.0):
                    raise RuntimeError("optfir could not generate an appropriate filter.")


class arb_resampler_ccf(gr.hier_block2):
    '''
    Convenience wrapper for the polyphase filterbank arbitrary resampler.

    The block takes a single complex stream in and outputs a single complex
    stream out. As such, it requires no extra glue to handle the input/output
    streams. This block is provided to be consistent with the interface to the
    other PFB block.
    '''
    def __init__(self, rate, taps=None, flt_size=32, atten=100):
        gr.hier_block2.__init__(self, "pfb_arb_resampler_ccf",
                                gr.io_signature(1, 1, gr.sizeof_gr_complex), # Input signature
                                gr.io_signature(1, 1, gr.sizeof_gr_complex)) # Output signature

        self._rate = rate
        self._size = flt_size

        if (taps is not None) and (len(taps) > 0):
            self._taps = taps
        else:
            self._taps = self.create_taps(self._rate, self._size, atten);

        self.pfb = filter.pfb_arb_resampler_ccf(self._rate, self._taps, self._size)
        #print("PFB has %d taps\n" % (len(self._taps),))

        self.connect(self, self.pfb)
        self.connect(self.pfb, self)

    # Note -- set_taps not implemented in base class yet
    def set_taps(self, taps):
        self.pfb.set_taps(taps)

    def set_rate(self, rate):
        self.pfb.set_rate(rate)

    def declare_sample_delay(self, delay):
        self.pfb.declare_sample_delay(delay)

    @staticmethod
    def create_taps(rate, flt_size=32, atten=100):
        # Create a filter that covers the full bandwidth of the output signal

        # If rate >= 1, we need to prevent images in the output,
        # so we have to filter it to less than half the channel
        # width of 0.5.  If rate < 1, we need to filter to less
        # than half the output signal's bw to avoid aliasing, so
        # the half-band here is 0.5*rate.
        percent = 0.80
        if(rate < 1):
            halfband = 0.5*rate
            bw = percent*halfband
            tb = (percent / 2.0)*halfband
            ripple = 0.1

            # As we drop the bw factor, the optfir filter has a harder time converging;
            # using the firdes method here for better results.
            return filter.firdes.low_pass_2(flt_size, flt_size, bw, tb, atten,
                                                  fft.window.WIN_BLACKMAN_HARRIS)
        else:
            halfband = 0.5
            bw = percent*halfband
            tb = (percent / 2.0)*halfband
            ripple = 0.1
            taps = None

            while True:
                try:
                    taps = optfir.low_pass(flt_size, flt_size, bw, bw+tb, ripple, atten)
                    return taps
                except RuntimeError:
                    ripple += 0.01
                    print("Warning: set ripple to %.4f dB. If this is a problem, adjust the attenuation or create your own filter taps." % (ripple))

                    # Build in an exit strategy; if we've come this far, it ain't working.
                    if(ripple >= 1.0):
                        raise RuntimeError("optfir could not generate an appropriate filter.")
        

class arb_resampler_fff(gr.hier_block2):
    '''
    Convenience wrapper for the polyphase filterbank arbitrary resampler.

    The block takes a single float stream in and outputs a single float
    stream out. As such, it requires no extra glue to handle the input/output
    streams. This block is provided to be consistent with the interface to the
    other PFB block.
    '''
    def __init__(self, rate, taps=None, flt_size=32, atten=100):
        gr.hier_block2.__init__(self, "pfb_arb_resampler_fff",
                                gr.io_signature(1, 1, gr.sizeof_float), # Input signature
                                gr.io_signature(1, 1, gr.sizeof_float)) # Output signature

        self._rate = rate
        self._size = flt_size

        if (taps is not None) and (len(taps) > 0):
            self._taps = taps
        else:
            self._taps = self.create_taps(self._rate, self._size, atten)

        self.pfb = filter.pfb_arb_resampler_fff(self._rate, self._taps, self._size)
        #print "PFB has %d taps\n" % (len(self._taps),)

        self.connect(self, self.pfb)
        self.connect(self.pfb, self)

    # Note -- set_taps not implemented in base class yet
    def set_taps(self, taps):
        self.pfb.set_taps(taps)

    def set_rate(self, rate):
        self.pfb.set_rate(rate)

    def declare_sample_delay(self, delay):
        self.pfb.declare_sample_delay(delay)

    @staticmethod
    def create_taps(rate, flt_size=32, atten=100):
        # Create a filter that covers the full bandwidth of the input signal

        # If rate >= 1, we need to prevent images in the output,
        # so we have to filter it to less than half the channel
        # width of 0.5.  If rate < 1, we need to filter to less
        # than half the output signal's bw to avoid aliasing, so
        # the half-band here is 0.5*rate.
        percent = 0.80
        if(rate < 1):
            halfband = 0.5*rate
            bw = percent*halfband
            tb = (percent / 2.0)*halfband
            ripple = 0.1

            # As we drop the bw factor, the optfir filter has a harder time converging;
            # using the firdes method here for better results.
            return filter.firdes.low_pass_2(flt_size, flt_size, bw, tb, atten,
                                                  fft.window.WIN_BLACKMAN_HARRIS)
        else:
            halfband = 0.5
            bw = percent*halfband
            tb = (percent / 2.0)*halfband
            ripple = 0.1

            while True:
                try:
                    taps = optfir.low_pass(flt_size, flt_size, bw, bw+tb, ripple, atten)
                    return taps
                except RuntimeError:
                    ripple += 0.01
                    print("Warning: set ripple to %.4f dB. If this is a problem, adjust the attenuation or create your own filter taps." % (ripple))

                    # Build in an exit strategy; if we've come this far, it ain't working.
                    if(ripple >= 1.0):
                        raise RuntimeError("optfir could not generate an appropriate filter.")

class arb_resampler_ccc(gr.hier_block2):
    '''
    Convenience wrapper for the polyphase filterbank arbitrary resampler.

    The block takes a single complex stream in and outputs a single complex
    stream out. As such, it requires no extra glue to handle the input/output
    streams. This block is provided to be consistent with the interface to the
    other PFB block.
    '''
    def __init__(self, rate, taps=None, flt_size=32, atten=100):
        gr.hier_block2.__init__(self, "pfb_arb_resampler_ccc",
                                gr.io_signature(1, 1, gr.sizeof_gr_complex), # Input signature
                                gr.io_signature(1, 1, gr.sizeof_gr_complex)) # Output signature

        self._rate = rate
        self._size = flt_size

        if (taps is not None) and (len(taps) > 0):
            self._taps = taps
        else:
            self._taps = self.create_taps(self._rate, self._size, atten)

        self.pfb = filter.pfb_arb_resampler_ccc(self._rate, self._taps, self._size)
        #print "PFB has %d taps\n" % (len(self._taps),)

        self.connect(self, self.pfb)
        self.connect(self.pfb, self)

    # Note -- set_taps not implemented in base class yet
    def set_taps(self, taps):
        self.pfb.set_taps(taps)

    def set_rate(self, rate):
        self.pfb.set_rate(rate)

    def declare_sample_delay(self, delay):
        self.pfb.declare_sample_delay(delay)

    @staticmethod
    def create_taps(rate, flt_size=32, atten=100):
        # Create a filter that covers the full bandwidth of the input signal
        bw = 0.4
        tb = 0.2
        ripple = 0.1
        while True:
            try:
                taps = optfir.low_pass(flt_size, flt_size, bw, bw+tb, ripple, atten)
                return taps
            except RuntimeError:
                ripple += 0.01
                print("Warning: set ripple to %.4f dB. If this is a problem, adjust the attenuation or create your own filter taps." % (ripple))

                # Build in an exit strategy; if we've come this far, it ain't working.
                if(ripple >= 1.0):
                    raise RuntimeError("optfir could not generate an appropriate filter.")

class channelizer_hier_ccf(gr.hier_block2):
    """
    Make a Polyphase Filter channelizer (complex in, complex out, floating-point taps)

    Args:
    n_chans: The number of channels to split into.
    n_filterbanks: The number of filterbank blocks to use (default=2).
    taps: The taps to use.  If this is `None` then taps are generated using optfir.low_pass.
    outchans: Which channels to output streams for (a list of integers) (default is all channels).
    atten: Stop band attenuation.
    bw: The fraction of the channel you want to keep.
    tb: Transition band with as fraction of channel width.
    ripple: Pass band ripple in dB.
    """

    def __init__(self, n_chans, n_filterbanks=1, taps=None, outchans=None,
                 atten=100, bw=1.0, tb=0.2, ripple=0.1):
        if n_filterbanks > n_chans:
            n_filterbanks = n_chans
        if outchans is None:
            outchans = list(range(n_chans))
        gr.hier_block2.__init__(
            self, "pfb_channelizer_hier_ccf",
            gr.io_signature(1, 1, gr.sizeof_gr_complex),
            gr.io_signature(len(outchans), len(outchans), gr.sizeof_gr_complex))
        if taps is None:
            taps = self.create_taps(n_chans, atten=100, bw=1.0, tb=0.2, ripple=0.1)
        taps = list(taps)
        extra_taps = int(math.ceil(1.0*len(taps)/n_chans)*n_chans - len(taps))
        taps = taps + [0] * extra_taps
        # Make taps for each channel
        chantaps = [list(reversed(taps[i: len(taps): n_chans])) for i in range(0, n_chans)]
        # Convert the input stream into a stream of vectors.
        self.s2v = blocks.stream_to_vector(gr.sizeof_gr_complex, n_chans)
        # Create a mapping to separate out each filterbank (a group of channels to be processed together)
        # And a list of sets of taps for each filterbank.
        low_cpp = int(n_chans / n_filterbanks)
        extra = n_chans - low_cpp*n_filterbanks
        cpps = [low_cpp+1]*extra + [low_cpp]*(n_filterbanks-extra)
        splitter_mapping = []
        filterbanktaps = []
        total = 0
        for cpp in cpps:
            splitter_mapping.append([(0, i) for i in range(total, total+cpp)])
            filterbanktaps.append(chantaps[total: total+cpp])
            total += cpp
        assert(total == n_chans)
        # Split the stream of vectors in n_filterbanks streams of vectors.
        self.splitter = blocks.vector_map(gr.sizeof_gr_complex, [n_chans], splitter_mapping)
        # Create the filterbanks
        self.fbs = [filter.filterbank_vcvcf(taps) for taps in filterbanktaps]
        # Combine the streams of vectors back into a single stream of vectors.
        combiner_mapping = [[]]
        for i, cpp in enumerate(cpps):
            for j in range(cpp):
                combiner_mapping[0].append((i, j))
        self.combiner = blocks.vector_map(gr.sizeof_gr_complex, cpps, combiner_mapping)
        # Add the final FFT to the channelizer.
        self.fft = fft.fft_vcc(n_chans, forward=True, window=[1.0]*n_chans)
        # Select the desired channels
        if outchans != list(range(n_chans)):
            selector_mapping = [[(0, i) for i in outchans]]
            self.selector = blocks.vector_map(gr.sizeof_gr_complex, [n_chans], selector_mapping)
        # Convert stream of vectors to a normal stream.
        self.v2ss = blocks.vector_to_streams(gr.sizeof_gr_complex, len(outchans))
        self.connect(self, self.s2v, self.splitter)
        for i in range(0, n_filterbanks):
            self.connect((self.splitter, i), self.fbs[i], (self.combiner, i))
        self.connect(self.combiner, self.fft)
        if outchans != list(range(n_chans)):
            self.connect(self.fft, self.selector, self.v2ss)
        else:
            self.connect(self.fft, self.v2ss)
        for i in range(0, len(outchans)):
            self.connect((self.v2ss, i), (self, i))

    @staticmethod
    def create_taps(n_chans, atten=100, bw=1.0, tb=0.2, ripple=0.1):
        return optfir.low_pass(1, n_chans, bw, bw+tb, ripple, atten)




#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

# The presence of this file turns this directory into a Python package

from .monitor import *

# import any pure python here
#import GNURadio



#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

'''
Filter blocks and related functions.
'''

import os
from gnuradio.fft import window

try:
    from .filter_python import *
except ImportError:
    dirname, filename = os.path.split(os.path.abspath(__file__))
    __path__.append(os.path.join(dirname, "bindings"))
    from .filter_python import *

from .filterbank import *
from .freq_xlating_fft_filter import *
from . import pfb
from . import optfir

# Pull this into the filter module
from .file_taps_loader import file_taps_loader



#
# Copyright 2005,2007,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from . import digital_python as digital
import struct

def gen_and_append_crc32(s):
    crc = digital.crc32(s)
    return s + struct.pack(">I", crc)

def check_crc32(s):
    if len(s) < 4:
        return (False, '')
    msg = s[:-4]
    #print "msg = '%s'" % (msg,)
    actual = digital.crc32(msg)
    (expected,) = struct.unpack(">I", s[-4:])
    # print "actual =", hex(actual), "expected =", hex(expected)
    return (actual == expected, msg)



#!D:\bld\gnuradio_1628795209068\_h_env\python.exe
#
# Copyright 2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr
from gnuradio import blocks
import sys
import numpy

def main():
    data = numpy.arange(0, 32000, 1).tolist()
    trig = 100*[0,] + 100*[1,]

    src = blocks.vector_source_s(data, True)
    trigger = blocks.vector_source_s(trig, True)

    thr = blocks.throttle(gr.sizeof_short, 10e3)
    ann = blocks.annotator_alltoall(1000000, gr.sizeof_short)
    tagger = blocks.burst_tagger(gr.sizeof_short)

    fsnk = blocks.tagged_file_sink(gr.sizeof_short, 1)

    tb = gr.top_block()
    tb.connect(src, thr, (tagger, 0))
    tb.connect(trigger, (tagger, 1))
    tb.connect(tagger, fsnk)

    tb.run()

if __name__ == "__main__":
    main()
